@@ -95,6 +95,8 @@ public:
 		DEF1_MODWHEEL,
 		DEF1_PRESSURE,
 		DEF1_VELOCITY,
+		DEF1_CHANNEL,
+		DEF1_MONO,
 
 		DCO2_SHAPE1,
 		DCO2_WIDTH1,
@@ -141,6 +143,8 @@ public:
 		DEF2_MODWHEEL,
 		DEF2_PRESSURE,
 		DEF2_VELOCITY,
+		DEF2_CHANNEL,
+		DEF2_MONO,
 
 		CHO1_WET,
 		CHO1_DELAY,
