@@ -1,7 +1,7 @@
 // synthv1_jack.h
 //
 /****************************************************************************
-   Copyright (C) 2012, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -49,11 +49,14 @@ public:
 
 	jack_client_t *client() const;
 
-	int process(jack_nframes_t nframes);
+	void open(const char *client_id);
+	void close();
 
 	void activate();
 	void deactivate();
 
+	int process(jack_nframes_t nframes);
+
 	void setParamValue(synthv1::ParamIndex index, float fValue);
 	float paramValue(synthv1::ParamIndex index) const;
 
@@ -62,11 +65,6 @@ public:
 	void alsa_capture(snd_seq_event_t *ev);
 #endif
 
-protected:
-
-	void open();
-	void close();
-
 private:
 
 	jack_client_t *m_client;
