@@ -20,6 +20,7 @@
 *****************************************************************************/
 
 #include "synthv1widget.h"
+#include "synthv1_param.h"
 
 #include <QDomDocument>
 #include <QTextStream>
@@ -29,132 +30,6 @@
 #include <QDir>
 
 
-//-------------------------------------------------------------------------
-// default state (params)
-
-static
-struct {
-
-	const char *name;
-	float value;
-
-} synthv1_default_params[synthv1::NUM_PARAMS] = {
-
-	{ "DCO1_SHAPE1",    1.0f },
-	{ "DCO1_WIDTH1",    1.0f },
-	{ "DCO1_SHAPE2",    1.0f },
-	{ "DCO1_WIDTH2",    1.0f },
-	{ "DCO1_BALANCE",   0.0f },
-	{ "DCO1_DETUNE",    0.1f },
-	{ "DCO1_PHASE",     0.0f },
-	{ "DCO1_OCTAVE",    0.0f },
-	{ "DCO1_TUNING",    0.0f },
-	{ "DCO1_GLIDE",     0.0f },
-	{ "DCF1_CUTOFF",    0.5f },
-	{ "DCF1_RESO",      0.0f },
-	{ "DCF1_TYPE",      0.0f },
-	{ "DCF1_SLOPE",     0.0f },
-	{ "DCF1_ENVELOPE",  1.0f },
-	{ "DCF1_ATTACK",    0.0f },
-	{ "DCF1_DECAY",     0.2f },
-	{ "DCF1_SUSTAIN",   0.5f },
-	{ "DCF1_RELEASE",   0.5f },
-	{ "LFO1_SHAPE",     1.0f },
-	{ "LFO1_WIDTH",     1.0f },
-	{ "LFO1_RATE",      0.5f },
-	{ "LFO1_SWEEP",     0.0f },
-	{ "LFO1_PITCH",     0.0f },
-	{ "LFO1_CUTOFF",    0.0f },
-	{ "LFO1_RESO",      0.0f },
-	{ "LFO1_PANNING",   0.0f },
-	{ "LFO1_VOLUME",    0.0f },
-	{ "LFO1_ATTACK",    0.0f },
-	{ "LFO1_DECAY",     0.1f },
-	{ "LFO1_SUSTAIN",   1.0f },
-	{ "LFO1_RELEASE",   0.5f },
-	{ "DCA1_VOLUME",    0.5f },
-	{ "DCA1_ATTACK",    0.0f },
-	{ "DCA1_DECAY",     0.1f },
-	{ "DCA1_SUSTAIN",   1.0f },
-	{ "DCA1_RELEASE",   0.1f },
-	{ "OUT1_WIDTH",     0.0f },
-	{ "OUT1_PANNING",   0.0f },
-	{ "OUT1_VOLUME",    0.5f },
-
-	{ "DEF1_PITCHBEND", 0.2f },
-	{ "DEF1_MODWHEEL",  0.2f },
-	{ "DEF1_PRESSURE",  0.2f },
-	{ "DEF1_VELOCITY",  0.2f },
-
-	{ "DCO2_SHAPE1",    1.0f },
-	{ "DCO2_WIDTH1",    1.0f },
-	{ "DCO2_SHAPE2",    1.0f },
-	{ "DCO2_WIDTH2",    1.0f },
-	{ "DCO2_BALANCE",   0.0f },
-	{ "DCO2_DETUNE",    0.1f },
-	{ "DCO2_PHASE",     0.0f },
-	{ "DCO2_OCTAVE",   -2.0f },
-	{ "DCO2_TUNING",    0.0f },
-	{ "DCO2_GLIDE",     0.0f },
-	{ "DCF2_CUTOFF",    0.5f },
-	{ "DCF2_RESO",      0.0f },
-	{ "DCF2_TYPE",      0.0f },
-	{ "DCF2_SLOPE",     0.0f },
-	{ "DCF2_ENVELOPE",  1.0f },
-	{ "DCF2_ATTACK",    0.0f },
-	{ "DCF2_DECAY",     0.2f },
-	{ "DCF2_SUSTAIN",   0.5f },
-	{ "DCF2_RELEASE",   0.5f },
-	{ "LFO2_SHAPE",     1.0f },
-	{ "LFO2_WIDTH",     1.0f },
-	{ "LFO2_RATE",      0.5f },
-	{ "LFO2_SWEEP",     0.0f },
-	{ "LFO2_PITCH",     0.0f },
-	{ "LFO2_CUTOFF",    0.0f },
-	{ "LFO2_RESO",      0.0f },
-	{ "LFO2_PANNING",   0.0f },
-	{ "LFO2_VOLUME",    0.0f },
-	{ "LFO2_ATTACK",    0.0f },
-	{ "LFO2_DECAY",     0.1f },
-	{ "LFO2_SUSTAIN",   1.0f },
-	{ "LFO2_RELEASE",   0.5f },
-	{ "DCA2_VOLUME",    0.5f },
-	{ "DCA2_ATTACK",    0.0f },
-	{ "DCA2_DECAY",     0.1f },
-	{ "DCA2_SUSTAIN",   1.0f },
-	{ "DCA2_RELEASE",   0.1f },
-	{ "OUT2_WIDTH",     0.0f },
-	{ "OUT2_PANNING",   0.0f },
-	{ "OUT2_VOLUME",    0.5f },
-
-	{ "DEF2_PITCHBEND", 0.2f },
-	{ "DEF2_MODWHEEL",  0.2f },
-	{ "DEF2_PRESSURE",  0.2f },
-	{ "DEF2_VELOCITY",  0.2f },
-
-	{ "CHO1_WET",       0.0f },
-	{ "CHO1_DELAY",     0.5f },
-	{ "CHO1_FEEDB",     0.5f },
-	{ "CHO1_RATE",      0.5f },
-	{ "CHO1_MOD",       0.5f },
-	{ "FLA1_WET",       0.0f },
-	{ "FLA1_DELAY",     0.5f },
-	{ "FLA1_FEEDB",     0.5f },
-	{ "FLA1_DAFT",      0.0f },
-	{ "PHA1_WET",       0.0f },
-	{ "PHA1_RATE",      0.5f },
-	{ "PHA1_FEEDB",     0.5f },
-	{ "PHA1_DEPTH",     0.5f },
-	{ "PHA1_DAFT",      0.0f },
-	{ "DEL1_WET",       0.0f },
-	{ "DEL1_DELAY",     0.5f },
-	{ "DEL1_FEEDB",     0.5f },
-	{ "DEL1_BPM",     180.0f },
-	{ "DYN1_COMPRESS",  0.0f },
-	{ "DYN1_LIMIT",     1.0f }
-};
-
-
 //-------------------------------------------------------------------------
 // synthv1widget - impl.
 //
@@ -188,7 +63,7 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	// Init swapable params A/B to default.
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i)
-		m_params_ab[i] = synthv1_default_params[i].value;
+		m_params_ab[i] = synthv1_param::paramDefaultValue(synthv1::ParamIndex(i));
 
 	// Start clean.
 	m_iUpdate = 0;
@@ -307,6 +182,19 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Lfo2VolumeKnob->setMinimum(-1.0f);
 	m_ui.Lfo2VolumeKnob->setMaximum(+1.0f);
 
+	// Channel filters
+	QStringList channels;
+	channels << tr("Omni");
+	for (int iChannel = 0; iChannel < 16; ++iChannel)
+		channels << QString::number(iChannel + 1);
+
+	m_ui.Def1ChannelKnob->insertItems(0, channels);
+	m_ui.Def2ChannelKnob->insertItems(0, channels);
+
+	// Mono switches
+	m_ui.Def1MonoKnob->insertItems(0, states);
+	m_ui.Def2MonoKnob->insertItems(0, states);
+
 	// Output (stereo-)width limits.
 	m_ui.Out1WidthKnob->setMinimum(-1.0f);
 	m_ui.Out1WidthKnob->setMaximum(+1.0f);
@@ -521,6 +409,8 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(synthv1::DEF1_MODWHEEL,  m_ui.Def1ModwheelKnob);
 	setParamKnob(synthv1::DEF1_PRESSURE,  m_ui.Def1PressureKnob);
 	setParamKnob(synthv1::DEF1_VELOCITY,  m_ui.Def1VelocityKnob);
+	setParamKnob(synthv1::DEF1_CHANNEL,   m_ui.Def1ChannelKnob);
+	setParamKnob(synthv1::DEF1_MONO,      m_ui.Def1MonoKnob);
 
 	// OUT1
 	setParamKnob(synthv1::OUT1_WIDTH,   m_ui.Out1WidthKnob);
@@ -722,6 +612,8 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(synthv1::DEF2_MODWHEEL,  m_ui.Def2ModwheelKnob);
 	setParamKnob(synthv1::DEF2_PRESSURE,  m_ui.Def2PressureKnob);
 	setParamKnob(synthv1::DEF2_VELOCITY,  m_ui.Def2VelocityKnob);
+	setParamKnob(synthv1::DEF2_CHANNEL,   m_ui.Def2ChannelKnob);
+	setParamKnob(synthv1::DEF2_MONO,      m_ui.Def2MonoKnob);
 
 	// OUT2
 	setParamKnob(synthv1::OUT2_WIDTH,   m_ui.Out2WidthKnob);
@@ -756,7 +648,6 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(synthv1::DYN1_COMPRESS, m_ui.Dyn1CompressKnob);
 	setParamKnob(synthv1::DYN1_LIMITER,  m_ui.Dyn1LimiterKnob);
 
-
 	// Preset management
 	QObject::connect(m_ui.Preset,
 		SIGNAL(newPresetFile()),
@@ -858,11 +749,17 @@ void synthv1widget::paramChanged ( float fValue )
 // Reset all param knobs to default values.
 void synthv1widget::resetParams (void)
 {
+	synthv1 *pSynth = instance();
+	if (pSynth == NULL)
+		return;
+
+	pSynth->reset();
+
 	resetSwapParams();
 
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
 		synthv1::ParamIndex index = synthv1::ParamIndex(i);
-		float fValue = synthv1_default_params[i].value;
+		float fValue = synthv1_param::paramDefaultValue(index);
 		synthv1widget_knob *pKnob = paramKnob(index);
 		if (pKnob)
 			fValue = pKnob->defaultValue();
@@ -921,7 +818,7 @@ void synthv1widget::resetParamValues (void)
 
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
 		synthv1::ParamIndex index = synthv1::ParamIndex(i);
-		float fValue = synthv1_default_params[i].value;
+		float fValue = synthv1_param::paramDefaultValue(index);
 		setParamValue(index, fValue);
 		updateParam(index, fValue);
 		m_params_ab[index] = fValue;
@@ -976,19 +873,27 @@ void synthv1widget::loadPreset ( const QString& sFilename )
 	qDebug("synthv1widget::loadPreset(\"%s\")", sFilename.toUtf8().constData());
 #endif
 
+	synthv1 *pSynth = instance();
+	if (pSynth == NULL)
+		return;
+
 	QFile file(sFilename);
 	if (!file.open(QIODevice::ReadOnly))
 		return;
 
 	static QHash<QString, synthv1::ParamIndex> s_hash;
 	if (s_hash.isEmpty()) {
-		for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i)
-			s_hash.insert(synthv1_default_params[i].name, synthv1::ParamIndex(i));
+		for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
+			synthv1::ParamIndex index = synthv1::ParamIndex(i);
+			s_hash.insert(synthv1_param::paramName(index), index);
+		}
 	}
 
 	resetParamValues();
 	resetParamKnobs();
 
+	pSynth->reset();
+
 	const QFileInfo fi(sFilename);
 	const QDir currentDir(QDir::current());
 	QDir::setCurrent(fi.absolutePath());
@@ -1059,10 +964,11 @@ void synthv1widget::savePreset ( const QString& sFilename )
 	QDomElement eParams = doc.createElement("params");
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
 		QDomElement eParam = doc.createElement("param");
+		synthv1::ParamIndex index = synthv1::ParamIndex(i);
 		eParam.setAttribute("index", QString::number(i));
-		eParam.setAttribute("name", synthv1_default_params[i].name);
-		eParam.appendChild(doc.createTextNode(QString::number(
-			paramValue(synthv1::ParamIndex(i)))));
+		eParam.setAttribute("name", synthv1_param::paramName(index));
+		eParam.appendChild(
+			doc.createTextNode(QString::number(paramValue(index))));
 		eParams.appendChild(eParam);
 	}
 	ePreset.appendChild(eParams);
